@@ -16,7 +16,7 @@
   information.
 
 */
-/* $Id: client_notify.c,v 1.93.2.1 2007/07/02 13:34:02 priikone Exp $ */
+/* $Id: client_notify.c,v 1.93.2.2 2007/09/01 08:52:01 priikone Exp $ */
 
 #include "silc.h"
 #include "silcclient.h"
@@ -56,9 +56,14 @@ static void silc_client_notify_resolved(SilcClient client,
   /* If entry is still invalid, resolving failed.  Finish notify processing. */
   if (notify->client_entry && !notify->client_entry->internal.valid) {
     /* If resolving timedout try it again many times. */
-    if (status != SILC_STATUS_ERR_TIMEDOUT || ++notify->resolve_retry > 1000)
+    if (status != SILC_STATUS_ERR_TIMEDOUT || ++notify->resolve_retry > 1000) {
       silc_fsm_next(notify->fsm, silc_client_notify_processed);
-    silc_client_unref_client(client, conn, notify->client_entry);
+
+      /* Unref client only in case of non-timeout error.  In case of timeout
+	 occurred, the routine reprocessing the notify is expected not to
+	 create new references of the entry. */
+      silc_client_unref_client(client, conn, notify->client_entry);
+    }
   }
 
   /* If no entries found, just finish the notify processing */
